<?php
// $Id: mew.admin.inc,v 1.1.2.8 2010/06/01 14:09:21 xano Exp $

/**
 * @file
 *   The Menu link wizard administration section.
 */

/**
 * Save an overridden menu item.
 *
 * @param $item array
 *   The menu item to save.
 *
 * @return mixed
 *   The return value of drupal_write_record().
 */
function mew_save(array $item) {
  mew_delete($item['path']);
  foreach (array('title', 'page', 'access') as $callback) {
    $item[$callback . '_arguments'] = serialize($item[$callback . '_arguments']);
  }

  return drupal_write_record('mew', $item);
}

/**
 * Delete an overwritten menu item.
 *
 * @param $path string
 *   The path of the menu item to delete.
 */
function mew_delete($path) {
  db_query("DELETE FROM {mew} WHERE path = '%s'", $path);
}

/**
 * List all available menu links.
 */
function mew_form_list(&$form_state) {
  drupal_add_css(drupal_get_path('module', 'mew') . '/css/mew.css');
  drupal_add_js(drupal_get_path('module', 'mew') . '/js/mew.js');

  $form['keywords'] = array(
    '#type' => 'textfield',
    '#title' => t('Filter menu links'),
    '#default_value' => isset($form_state['values']) ? $form_state['values']['keywords'] : NULL,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
    '#attributes' => array(
      'class' => 'js-hide',
    ),
  );
  $keywords = array();
  if (isset($form_state['values']['keywords']) && strlen($form_state['values']['keywords'])) {
    $keywords = explode(' ', $form_state['values']['keywords']);
  }
  $form['list'] = array(
    '#value' => mew_list($keywords),
    '#prefix' => '<div id ="mew-list-wrapper">',
    '#suffix' => '</div>',
  );

  return $form;
}

/**
 * Form submit handler for mew_form_list().
 */
function mew_form_list_submit(array $form, array &$form_state) {
  $form_state['rebuild'] = TRUE;
}

/**
 * List all available menu links.
 */
function mew_list(array $keywords = array()) {
  $items = mew_menu_items();
  $overwritten_paths= array();
  $result = db_query("SELECT path FROM {mew}");
  while ($path = db_result($result)) {
    $overwritten_paths[] = $path;
  }

  // Sort the items.
  ksort($items);
  if ($_GET['order'] == t('Link title')) {
    uasort($items, 'mew_sort_title');
  }
  if ($_GET['sort'] == 'desc') {
    $items = array_reverse($items);
  }

  $rows = array();
 	foreach ($items as $path => $item) {
  	if ($keywords) {
			$strings = array($path, $item['title']);
	  	$matches = array(TRUE, TRUE);
			foreach ($strings as $i => $string) {
	     	foreach ($keywords as $keyword) {
	       	if (stripos($string, $keyword) === FALSE) {
	         	$matches[$i] = FALSE;
					}
	       }
			}
			// Not all keywords are present, so skip this menu link.
			if (!$matches[0] && !$matches[1]) {
				continue;
			}
		}

    $reset = $class = NULL;
    if (in_array($path, $overwritten_paths)) {
      $class = 'mew-overwritten';
      $reset = l(t('reset'), 'admin/build/mew/reset/' . $path);
    }
    $rows[] = array(
      array(
        'data' => $path,
        'class' => $class,
      ),
      array(
        'data' => $item['title'],
        'class' => $class,
      ),
      l(t('edit'), 'admin/build/mew/edit/' . $path),
      $reset,
    );
  }

  $header = array(
    array(
      'data' => t('URL path'),
      'field' => t('path'),
    ),
    array(
      'data' => t('Link title'),
      'field' => t('title'),
    ),
    array(
      'data' => t('Operations'),
      'colspan' => 2,
    ),
  );

  return theme('table', $header, $rows, array('id' => 'mew-list'));
}

/**
 * Sort menu items by title. usort() callback.
 */
function mew_sort_title(array $item_a, array $item_b) {
  return strcmp($item_a['title'], $item_b['title']);
}

/**
 * Menu link edit form.
 */
function mew_form_edit() {
  // The first 22 characters are "admin/build/mew/edit/", which is the menu
  // item for this form.
  $path = substr($_GET['q'], 21);
  if (!mew_menu_item($path)) {
    drupal_not_found();
    exit;
  }

  $item = mew_menu_item($path);

  drupal_set_title(t('Edit %url_path', array('%url_path' => $path)));

  $form['#redirect'] = 'admin/build/mew';
  $form['path'] = array(
    '#type' => 'value',
    '#value' => $path,
  );
  $form['type'] = array(
    '#type' => 'select',
    '#title' => t('Link type'),
    '#options' => array(
      MENU_NORMAL_ITEM => t('Normal'),
      MENU_LOCAL_TASK => t('Tab (local task)'),
      MENU_DEFAULT_LOCAL_TASK => t('Default tab (local task)'),
      MENU_CALLBACK => t('Hidden (callback)'),
    ),
    '#default_value' => $item['type'],
  );
  $form['callbacks'] = array(
    '#type' => 'fieldset',
    '#title' => t('Callback information'),
    '#description' => '<div class="warning">' . t('Entering incorrect function names or arguments may cause your website to malfunction. In case of trouble, uninstall Menu Wizard.') . '</div',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['callbacks']['title_callback'] = array(
    '#type' => 'textfield',
    '#title' => t('Title callback'),
    '#description' => t('Defaults to !t.', array('!t' => l('t()', 'http://api.drupal.org/api/function/t/6'))),
    '#default_value' => $item['title_callback'],
  );
  $form['callbacks']['title_arguments'] = array(
    '#type' => 'textarea',
    '#title' => t('Title arguments'),
    '#description' => t("One argument per line. Integers will match the corresponding URL path component. The menu item's title will automatically be the first argument."),
    '#default_value' => implode("\n", $item['title_arguments']),
  );
  $form['callbacks']['page_callback'] = array(
    '#type' => 'textfield',
    '#title' => t('Page callback'),
    '#default_value' => $item['page_callback'],
    '#required' => TRUE,
  );
  $form['callbacks']['page_arguments'] = array(
    '#type' => 'textarea',
    '#title' => t('Page arguments'),
    '#description' => t('One argument per line. Integers will match the corresponding URL path component.'),
    '#default_value' => implode("\n", $item['page_arguments']),
  );
  $form['callbacks']['access_callback'] = array(
    '#type' => 'textfield',
    '#title' => t('Access callback'),
    '#description' => t('Enter <code>TRUE</code> to allow access to everyone.'),
    '#default_value' => $item['access_callback'],
    '#required' => TRUE,
  );
  $form['callbacks']['access_arguments'] = array(
    '#type' => 'textarea',
    '#title' => t('Access arguments'),
    '#description' => t('One argument per line. Integers will match the corresponding URL path component.'),
    '#default_value' => implode("\n", $item['access_arguments']),
  );
  $form['callbacks']['file'] = array(
    '#type' => 'textfield',
    '#title' => t('File'),
    '#default_value' => $item['file'],
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  if (db_result(db_query("SELECT COUNT(*) FROM {mew} WHERE path = '%s'", $path))) {
    $form['buttons']['reset'] = array(
      '#type' => 'markup',
      '#value' => l(t('Reset'), 'admin/build/mew/reset/' . $path),
    );
  }

  return $form;
}

/**
 * Submit callback for mew_form_edit().
 */
function mew_form_edit_validate(array $form, array &$form_state) {
  $values = $form_state['values'];
  // Confirm that callbacks exist.
  if ($values['file']) {
    include($values['file']);
  }
  if (!empty($values['title_callback'])) {
    if (!function_exists($values['title_callback'])) {
      form_set_error('title_callback', t('The title callback cannot be found.'));
    }
    $reflection = new ReflectionFunction($values['title_callback']);
    $count = $reflection->getNumberOfRequiredParameters() - 1;
    if (count(preg_split('#\v#', $values['title_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
      form_set_error('title_arguments', t('The title callback requires at least !count arguments.', array('!count' => $count)));
    }
  }
  if (!empty($values['page_callback'])) {
    if (!function_exists($values['page_callback'])) {
      form_set_error('page_callback', t('The page callback cannot be found.'));
    }
    $reflection = new ReflectionFunction($values['page_callback']);
    $count = $reflection->getNumberOfRequiredParameters();
    if (count(preg_split('#\v#', $values['page_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
      form_set_error('page_arguments', t('The page callback requires at least !count arguments.', array('!count' => $count)));
    }
  }
  if (!empty($values['access_callback'])) {
    if (!function_exists($values['access_callback'])) {
      form_set_error('access_callback', t('The access callback cannot be found.'));
    }
    $reflection = new ReflectionFunction($values['access_callback']);
    $count = $reflection->getNumberOfRequiredParameters();
    if (count(preg_split('#\v#', $values['access_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
      form_set_error('access_arguments', t('The access callback requires at least !count arguments.', array('!count' => $count)));
    }
  }
}

/**
 * Submit callback for mew_form_edit().
 */
function mew_form_edit_submit(array $form, array &$form_state) {
  $item = $form_state['values'];
  // The title callback defaults to t().
  if (empty($item['title_callback'])) {
    $item['title_callback'] = 't';
  }
  // The access callback can be TRUE to allow access to everyone.
  if ($item['access_callback'] == 'TRUE') {
    $item['access_callback'] = TRUE;
  }
  // Convert callback arguments to arrays.
  foreach (array('title', 'page', 'access') as $callback) {
    $item[$callback . '_arguments'] = preg_split('#\v#', $item[$callback . '_arguments'], NULL, PREG_SPLIT_NO_EMPTY);
    // Convert numbers to integers, so they match URL path components.
    foreach ($item[$callback . '_arguments'] as &$argument) {
      if (!preg_match('#\D#', $argument)) {
        $argument = (int) $argument;
      }
    }
  }
  mew_save($item);
  mew_menu_rebuild();
  drupal_set_message(t('The menu item for %path has been overwritten.', array('%path' => $item['path'])));
}

/**
 * Form: reset a menu link.
 *
 * @return array
 *   A Drupal form.
 */
function mew_form_reset(array &$form_state) {
  // The first 23 characters are "admin/build/mew/reset", which is the menu
  // item for this form.
  $path = substr($_GET['q'], 22);

  if (!mew_menu_item($path)) {
    drupal_not_found();
    exit;
  }

  $form['#redirect'] = 'admin/build/mew';
  $form['path'] = array(
    '#type' => 'value',
    '#value' => $path,
  );

  return confirm_form($form,
    t('Are you sure you want to reset %title?', array('%title' => $path)),
    isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/mew',
    t('This action cannot be undone.'),
    t('Reset'),
    t('Cancel')
  );
}

/**
 * Submit callback for mew_form_reset().
 */
function mew_form_reset_submit(array $form, array &$form_state) {
  mew_delete($form_state['values']['path']);
  mew_menu_rebuild(); 
  drupal_set_message(t('The menu item for %path has been reset.', array('%path' => $form_state['values']['path'])));
}

/**
 * Rebuild the menu after changing menu items.
 */
function mew_menu_rebuild() {
  // Menu items can appear twice after overwriting them. The following
  // workaround prevents this: disable all modules, rebuild the menu, enable
  // all modules and rebuild the menu again. This doesn't work if all modules
  // are disabled, because then there are no menu items, so we do this
  // separately for Menu Wizard.
  // Get all currently set messages so we can set them again later to hide
  // messages that are set when enabling modules.
  $messages = drupal_get_messages();
  $modules = module_list();
  unset($modules['mew']);
  module_disable($modules);
  menu_rebuild();
  module_enable($modules);
  menu_rebuild();
  module_disable(array('mew'));
  menu_rebuild();
  module_enable(array('mew'));
  menu_rebuild();
  // Clear messages that were set in the meantime and set the old ones again.
  drupal_get_messages();
  foreach ($messages as $type => $type_messages) {
    foreach ($type_messages as $message) {
      drupal_set_message($message, $type);
    }
  }
}